@@ -7,7 +7,7 @@
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
 #  Sign up for free at http://www.comet.ml
-#  Copyright (C) 2015-2020 Comet ML INC
+#  Copyright (C) 2015-2021 Comet ML INC
 #  This file can not be copied and/or distributed without the express
 #  permission of Comet ML Inc.
 # *******************************************************
@@ -29,7 +29,11 @@ from ._logging import (
     setup,
     setup_http_handler,
 )
-from ._reporting import EXPERIMENT_CREATED, EXPERIMENT_CREATION_FAILED
+from ._reporting import (
+    EXPERIMENT_CREATED,
+    EXPERIMENT_CREATION_DURATION,
+    EXPERIMENT_CREATION_FAILED,
+)
 from ._typing import Any, Dict, Optional
 from .api import API, APIExperiment
 from .comet import Streamer, format_url, generate_guid, is_valid_experiment_key
@@ -46,6 +50,7 @@ from .connection import (
     RestServerConnection,
     WebSocketConnection,
     get_backend_address,
+    get_comet_api_client,
     get_rest_api_client,
     log_url,
 )
@@ -55,8 +60,10 @@ from .feature_toggles import HTTP_LOGGING, FeatureToggles
 from .json_encoder import NestedEncoder
 from .loggers.fastai_logger import patch as fastai_patch
 from .loggers.keras_logger import patch as keras_patch
+from .loggers.lightgbm_logger import patch as lgbm_patch
 from .loggers.mlflow_logger import patch as mlflow_patch
 from .loggers.pytorch_logger import patch as pytorch_patch
+from .loggers.shap_logger import patch as shap_patch
 from .loggers.sklearn_logger import patch as sklearn_patch
 from .loggers.tensorboard_logger import patch as tb_patch
 from .loggers.tensorflow_logger import patch as tf_patch
@@ -67,7 +74,14 @@ from .monkey_patching import CometModuleFinder
 from .offline import OfflineExperiment
 from .optimizer import Optimizer
 from .rpc import create_remote_call, get_remote_action_definition
-from .utils import Embedding, Histogram, get_comet_version, merge_url, valid_ui_tabs
+from .utils import (
+    Embedding,
+    Histogram,
+    get_comet_version,
+    get_time_monotonic,
+    merge_url,
+    valid_ui_tabs,
+)
 
 __author__ = "Gideon<Gideon@comet.ml>"
 __all__ = [
@@ -77,6 +91,7 @@ __all__ = [
     "Embedding",
     "ExistingExperiment",
     "Experiment",
+    "get_comet_api_client",
     "Histogram",
     "OfflineExperiment",
     "Optimizer",
@@ -97,6 +112,8 @@ if not get_config("comet.disable_auto_logging"):
     mlflow_patch(MODULE_FINDER)
     xg_patch(MODULE_FINDER)
     tfma_patch(MODULE_FINDER)
+    shap_patch(MODULE_FINDER)
+    lgbm_patch(MODULE_FINDER)
     MODULE_FINDER.start()
 
 # Configure the logging
@@ -124,27 +141,33 @@ class Experiment(BaseExperiment):
 
     def __init__(
         self,
-        api_key=None,
-        project_name=None,
-        workspace=None,
-        log_code=True,
-        log_graph=True,
-        auto_param_logging=True,
-        auto_metric_logging=True,
-        parse_args=True,
-        auto_output_logging="default",
-        log_env_details=True,
-        log_git_metadata=True,
-        log_git_patch=True,
-        disabled=False,
-        log_env_gpu=True,
-        log_env_host=True,
-        display_summary=None,
-        log_env_cpu=True,
-        display_summary_level=1,
-        optimizer_data=None,
-        auto_weight_logging=False,
-        auto_log_co2=True,
+        api_key=None,  # type: Optional[str]
+        project_name=None,  # type: Optional[str]
+        workspace=None,  # type: Optional[str]
+        log_code=True,  # type: Optional[bool]
+        log_graph=True,  # type: Optional[bool]
+        auto_param_logging=True,  # type: Optional[bool]
+        auto_metric_logging=True,  # type: Optional[bool]
+        parse_args=True,  # type: Optional[bool]
+        auto_output_logging="default",  # type: Optional[str]
+        log_env_details=True,  # type: Optional[bool]
+        log_git_metadata=True,  # type: Optional[bool]
+        log_git_patch=True,  # type: Optional[bool]
+        disabled=False,  # type: Optional[bool]
+        log_env_gpu=True,  # type: Optional[bool]
+        log_env_host=True,  # type: Optional[bool]
+        display_summary=None,  # type: Optional[bool]
+        log_env_cpu=True,  # type: Optional[bool]
+        display_summary_level=1,  # type: Optional[int]
+        optimizer_data=None,  # type: Optional[Dict[str, Any]]
+        auto_weight_logging=None,  # type: Optional[bool]
+        auto_log_co2=True,  # type: Optional[bool]
+        auto_metric_step_rate=10,  # type: Optional[int]
+        auto_histogram_tensorboard_logging=False,  # type: Optional[bool]
+        auto_histogram_epoch_rate=1,  # type: Optional[int]
+        auto_histogram_weight_logging=False,  # type: Optional[bool]
+        auto_histogram_gradient_logging=False,  # type: Optional[bool]
+        auto_histogram_activation_logging=False,  # type: Optional[bool]
     ):
         """
         Creates a new experiment on the Comet.ml frontend.
@@ -157,7 +180,12 @@ class Experiment(BaseExperiment):
             log_graph: Default(True) - allows you to enable/disable automatic computation graph logging.
             auto_param_logging: Default(True) - allows you to enable/disable hyper parameters logging
             auto_metric_logging: Default(True) - allows you to enable/disable metrics logging
-            auto_weight_logging: Default(False) - allows you to enable/disable histogram logging for bias and weights
+            auto_metric_step_rate: Default(10) - controls how often batch metrics are logged
+            auto_histogram_tensorboard_logging: Default(False) - allows you to enable/disable automatic tensorboard histogram logging
+            auto_histogram_epoch_rate: Default(1) - controls how often histograms are logged
+            auto_histogram_weight_logging: Default(False) - allows you to enable/disable histogram logging for biases and weights
+            auto_histogram_gradient_logging: Default(False) - allows you to enable/disable automatic histogram logging of gradients
+            auto_histogram_activation_logging: Default(False) - allows you to enable/disable automatic histogram logging of activations
             auto_output_logging: Default("default") - allows you to select
                 which output logging mode to use. You can pass `"native"`
                 which will log all output even when it originated from a C
@@ -193,6 +221,7 @@ class Experiment(BaseExperiment):
                 want to test to make sure everything is working, without actually
                 logging anything.
         """
+        self._startup_duration_start = get_time_monotonic()
         self.config = get_config()
 
         self.api_key = get_api_key(api_key, self.config)
@@ -219,14 +248,21 @@ class Experiment(BaseExperiment):
             disabled=disabled,
             log_env_gpu=log_env_gpu,
             log_env_host=log_env_host,
+            display_summary=display_summary,  # deprecated
             display_summary_level=display_summary_level,
             log_env_cpu=log_env_cpu,
             optimizer_data=optimizer_data,
-            auto_weight_logging=auto_weight_logging,
+            auto_weight_logging=auto_weight_logging,  # deprecated
             auto_log_co2=auto_log_co2,
+            auto_metric_step_rate=auto_metric_step_rate,
+            auto_histogram_tensorboard_logging=auto_histogram_tensorboard_logging,
+            auto_histogram_epoch_rate=auto_histogram_epoch_rate,
+            auto_histogram_weight_logging=auto_histogram_weight_logging,
+            auto_histogram_gradient_logging=auto_histogram_gradient_logging,
+            auto_histogram_activation_logging=auto_histogram_activation_logging,
         )
 
-        self.ws_connection = None
+        self.ws_connection = None  # type: Optional[WebSocketConnection]
         self.connection = None  # type: Optional[RestServerConnection]
         self.rest_api_client = None
 
@@ -236,6 +272,12 @@ class Experiment(BaseExperiment):
             if self.alive is True:
                 self._report(event_name=EXPERIMENT_CREATED)
 
+                startup_duration = get_time_monotonic() - self._startup_duration_start
+                self._report(
+                    event_name=EXPERIMENT_CREATION_DURATION,
+                    err_msg=str(startup_duration),
+                )
+
                 LOGGER.info(EXPERIMENT_LIVE, self._get_experiment_url())
 
     def _setup_http_handler(self):
@@ -258,7 +300,12 @@ class Experiment(BaseExperiment):
         self.connection = RestServerConnection(
             self.api_key, self.id, server_address, self.config["comet.timeout.http"]
         )
-        self.rest_api_client = get_rest_api_client("v2", api_key=self.api_key)
+        self.rest_api_client = get_rest_api_client(
+            "v2",
+            api_key=self.api_key,
+            use_cache=False,
+            headers={"X-COMET-SDK-SOURCE": "Experiment"},
+        )
 
         try:
             results = self._authenticate()
@@ -716,10 +763,11 @@ class ExistingExperiment(Experiment):
 
     def __init__(self, api_key=None, previous_experiment=None, **kwargs):
         """
-        Creates a new experiment on the Comet.ml frontend.
+        Append to an existing experiment on the Comet.ml frontend.
         Args:
             api_key: Your API key obtained from comet.ml
-            previous_experiment: Your experiment key from comet.ml
+            previous_experiment: Optional. Your experiment key from comet.ml, could be set through
+                configuration as well.
             project_name: Optional. Send your experiment to a specific project. Otherwise will be sent to `Uncategorized Experiments`.
                              If project name does not already exists Comet.ml will create a new project.
             workspace: Optional. Attach an experiment to a project that belongs to this workspace
@@ -727,7 +775,12 @@ class ExistingExperiment(Experiment):
             log_graph: Default(False) - allows you to enable/disable automatic computation graph logging.
             auto_param_logging: Default(True) - allows you to enable/disable hyper parameters logging
             auto_metric_logging: Default(True) - allows you to enable/disable metrics logging
-            auto_weight_logging: Default(False) - allows you to enable/disable hisogram logging for bias and weights
+            auto_metric_step_rate: Default(10) - controls how often batch metrics are logged
+            auto_histogram_tensorboard_logging: Default(False) - allows you to enable/disable automatic tensorboard histogram logging
+            auto_histogram_epoch_rate: Default(1) - controls how often histograms are logged
+            auto_histogram_weight_logging: Default(False) - allows you to enable/disable automatic histogram logging of biases and weights
+            auto_histogram_gradient_logging: Default(False) - allows you to enable/disable automatic histogram logging of gradients
+            auto_histogram_activation_logging: Default(False) - allows you to enable/disable automatic histogram logging of activations
             auto_output_logging: Default("default") - allows you to select
                 which output logging mode to use. You can pass `"native"`
                 which will log all output even when it originated from a C
@@ -777,7 +830,7 @@ class ExistingExperiment(Experiment):
         * log_env_cpu
         * log_env_host
 
-        For example, to contiue to collect GPU information in an
+        For example, to continue to collect GPU information in an
         `ExistingExperiment` you will need to override these parameters:
 
         ```python
@@ -796,6 +849,7 @@ class ExistingExperiment(Experiment):
         if not is_valid_experiment_key(self.previous_experiment):
             raise ValueError("Invalid experiment key: %s" % self.previous_experiment)
 
+        # TODO: Document the parameter
         self.step_copy = kwargs.pop("step_copy", None)
 
         ## Defaults for ExistingExperiment:
