@@ -7,7 +7,7 @@
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
 #  Sign up for free at http://www.comet.ml
-#  Copyright (C) 2015-2020 Comet ML INC
+#  Copyright (C) 2015-2021 Comet ML INC
 #  This file can not be copied and/or distributed without the express
 #  permission of Comet ML Inc.
 # *******************************************************
@@ -19,7 +19,6 @@ This module contains the main components of comet.ml client side
 
 """
 import abc
-import json
 import logging
 import os
 import shutil
@@ -34,11 +33,17 @@ from six.moves.queue import Empty, Queue
 from six.moves.urllib.parse import urlencode, urlsplit, urlunsplit
 
 from ._reporting import ON_EXIT_DIDNT_FINISH_UPLOAD_SDK
-from ._typing import List, Optional, Tuple
+from ._typing import Any, Dict, List, Optional, Tuple
+from .compat_utils import json_dump
 from .config import ADDITIONAL_STREAMER_UPLOAD_TIMEOUT, DEFAULT_STREAMER_MSG_TIMEOUT
 from .connection import format_messages_for_ws
 from .exceptions import CometRestApiException
-from .file_uploader import is_user_text, upload_file_like_thread, upload_file_thread
+from .file_uploader import (
+    is_user_text,
+    upload_file_like_thread,
+    upload_file_thread,
+    upload_remote_asset_thread,
+)
 from .json_encoder import NestedEncoder
 from .logging_messages import (
     CLOUD_DETAILS_MSG_SENDING_ERROR,
@@ -53,6 +58,7 @@ from .messages import (
     Message,
     ModelGraphMessage,
     OsPackagesMessage,
+    RemoteAssetMessage,
     SystemDetailsMessage,
     UploadFileMessage,
     UploadInMemoryMessage,
@@ -244,6 +250,8 @@ class Streamer(BaseStreamer):
                             self._process_upload_message(message)
                         elif isinstance(message, UploadInMemoryMessage):
                             self._process_upload_in_memory_message(message)
+                        elif isinstance(message, RemoteAssetMessage):
+                            self._process_upload_remote_asset_message(message)
                         elif isinstance(message, Message):
                             self._process_ws_message(message, offset)
                         elif isinstance(message, OsPackagesMessage):
@@ -305,6 +313,24 @@ class Streamer(BaseStreamer):
         LOGGER.debug("Processing uploading message done")
         LOGGER.debug("Upload threads %s", self.upload_threads)
 
+    def _process_upload_remote_asset_message(self, message):
+        # type: (RemoteAssetMessage) -> None
+        # Compute the url from the upload type
+        url = self.connection.get_upload_url(message.upload_type)
+
+        upload_thread = upload_remote_asset_thread(
+            project_id=self.project_id,
+            experiment_id=self.experiment_key,
+            remote_uri=message.remote_uri,
+            upload_endpoint=url,
+            api_key=self.api_key,
+            additional_params=message.additional_params,
+            metadata=message.metadata,
+        )
+        self.upload_threads.append(upload_thread)
+        LOGGER.debug("Processing uploading message done")
+        LOGGER.debug("Upload threads %s", self.upload_threads)
+
     def _process_ws_message(self, message, offset):
         # type: (Message, int) -> None
         try:
@@ -456,16 +482,21 @@ class Streamer(BaseStreamer):
                     LOGGER.debug("Error calling the rpc callback", exc_info=True)
 
     def wait_for_finish(self):
-        """ Blocks the experiment from exiting until all data was sent to server OR 2 minutes passed."""
+        """ Blocks the experiment from exiting until all data was sent to server OR the configured timeouts has expired."""
 
-        msg = "Uploading stats to Comet before program termination (may take several seconds)"
+        msg = "Uploading metrics, params, and assets to Comet before program termination (may take several seconds)"
         if not self._is_msg_queue_empty():
             log_once_at_level(logging.INFO, msg)
+            log_once_at_level(
+                logging.INFO,
+                "The Python SDK has %d seconds to finish before aborting...",
+                self.msg_waiting_timeout,
+            )
 
             wait_for_empty(
                 self._is_msg_queue_empty,
                 self.msg_waiting_timeout,
-                verbose=True,
+                progress_callback=self._show_remaining_messages,
                 sleep_time=5,
             )
 
@@ -482,10 +513,14 @@ class Streamer(BaseStreamer):
                 "Waiting for completion of the file uploads (may take several seconds)"
             )
             LOGGER.info(msg)
+            LOGGER.info(
+                "The Python SDK has %d seconds to finish before aborting...",
+                self.file_upload_timeout,
+            )
             wait_for_empty(
                 self._is_file_upload_done,
                 self.file_upload_timeout,
-                verbose=True,
+                progress_callback=self._show_remaining_file_uploads,
                 sleep_time=5,
             )
 
@@ -520,6 +555,10 @@ class Streamer(BaseStreamer):
 
         return finished
 
+    def _show_remaining_messages(self):
+        remaining = self.messages.qsize()
+        LOGGER.info("Uploading %d metrics, params and output messages", remaining)
+
     def _is_file_upload_done(self):
         finished = True
 
@@ -529,9 +568,19 @@ class Streamer(BaseStreamer):
 
         return finished
 
+    def _show_remaining_file_uploads(self):
+        remaining_upload_number = 0
+        for thread in self.upload_threads:
+            if thread.is_alive() is True:
+                remaining_upload_number += 1
 
-def compact_json(data):
-    return json.dumps(data, sort_keys=True, separators=(",", ":"), cls=NestedEncoder)
+        LOGGER.info(
+            "Still uploading %d file(s)", remaining_upload_number,
+        )
+
+
+def compact_json_dump(data, fp):
+    return json_dump(data, fp, sort_keys=True, separators=(",", ":"), cls=NestedEncoder)
 
 
 class OfflineStreamer(BaseStreamer):
@@ -546,7 +595,7 @@ class OfflineStreamer(BaseStreamer):
         self.closed = False
         self.tmp_dir = tmp_dir
 
-        self.file = open(os.path.join(self.tmp_dir, "messages.json"), "w")
+        self.file = open(os.path.join(self.tmp_dir, "messages.json"), "wb")
 
     def close(self):
         """
@@ -566,6 +615,12 @@ class OfflineStreamer(BaseStreamer):
 
         LOGGER.debug("OfflineStreamer has been closed")
 
+    def _write(self, json_line_message):
+        # type: (Dict[str, Any]) -> None
+        compact_json_dump(json_line_message, self.file)
+        self.file.write(b"\n")
+        self.file.flush()
+
     def _after_run(self):
         # Close the messages files once we are sure we won't write in it
         # anymore
@@ -602,6 +657,8 @@ class OfflineStreamer(BaseStreamer):
                         self._process_system_details_message(message)
                     elif isinstance(message, CloudDetailsMessage):
                         self._process_cloud_details_message(message)
+                    elif isinstance(message, RemoteAssetMessage):
+                        self._process_remote_asset_message(message)
                     else:
                         raise ValueError("Unkown message type %r", message)
 
@@ -638,8 +695,7 @@ class OfflineStreamer(BaseStreamer):
 
         msg_json = message.repr_json()
         data = {"type": "file_upload", "payload": msg_json}
-        self.file.write(compact_json(data) + "\n")
-        self.file.flush()
+        self._write(data)
 
     def _process_upload_in_memory_message(self, message):
         # type: (UploadInMemoryMessage) -> None
@@ -667,40 +723,42 @@ class OfflineStreamer(BaseStreamer):
         msg_json = message._non_null_dict()
 
         data = {"type": "ws_msg", "payload": msg_json}
-        self.file.write(compact_json(data) + "\n")
-        self.file.flush()
+        self._write(data)
 
     def _process_os_package_message(self, message):
         # type: (OsPackagesMessage) -> None
         msg_json = message._non_null_dict()
 
         data = {"type": "os_packages", "payload": msg_json}
-        self.file.write(compact_json(data) + "\n")
-        self.file.flush()
+        self._write(data)
 
     def _process_model_graph_message(self, message):
         # type: (ModelGraphMessage) -> None
         msg_json = message._non_null_dict()
 
         data = {"type": "graph", "payload": msg_json}
-        self.file.write(compact_json(data) + "\n")
-        self.file.flush()
+        self._write(data)
 
     def _process_system_details_message(self, message):
         # type: (SystemDetailsMessage) -> None
         msg_json = message._non_null_dict()
 
         data = {"type": "system_details", "payload": msg_json}
-        self.file.write(compact_json(data) + "\n")
-        self.file.flush()
+        self._write(data)
 
     def _process_cloud_details_message(self, message):
         # type: (CloudDetailsMessage) -> None
         msg_json = message._non_null_dict()
 
         data = {"type": "cloud_details", "payload": msg_json}
-        self.file.write(compact_json(data) + "\n")
-        self.file.flush()
+        self._write(data)
+
+    def _process_remote_asset_message(self, message):
+        # type: (RemoteAssetMessage) -> None
+        msg_json = message._non_null_dict()
+
+        data = {"type": "remote_file", "payload": msg_json}
+        self._write(data)
 
     def wait_for_finish(self):
         """ Blocks the experiment from exiting until all data was sent to server OR 30 seconds passed."""
