@@ -7,7 +7,7 @@
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
 #  Sign up for free at http://www.comet.ml
-#  Copyright (C) 2015-2020 Comet ML INC
+#  Copyright (C) 2015-2021 Comet ML INC
 #  This file can not be copied and/or distributed without the express
 #  permission of Comet ML Inc.
 # *******************************************************
@@ -31,7 +31,8 @@ from everett.manager import (
     parse_bool,
 )
 
-from ._typing import Any, Dict, Optional
+from ._typing import Any, Dict, List, Optional
+from .utils import log_once_at_level
 
 try:  # everett version 1.0.1 or greater
     from everett.ext.inifile import ConfigIniEnv
@@ -56,13 +57,13 @@ DEBUG = False
 # Global experiment placeholder. Should be set by the latest call of Experiment.init()
 experiment = None
 
-DEFAULT_UPLOAD_SIZE_LIMIT = 100 * 1024 * 1024  # 100 MebiBytes
+DEFAULT_UPLOAD_SIZE_LIMIT = 200 * 1024 * 1024  # 100 MebiBytes
 
-DEFAULT_ASSET_UPLOAD_SIZE_LIMIT = 2500 * 1024 * 1024  # 2.5GB
+DEFAULT_ASSET_UPLOAD_SIZE_LIMIT = 50000 * 1024 * 1024  # 2.5GB
 
-DEFAULT_STREAMER_MSG_TIMEOUT = 5 * 60
+DEFAULT_STREAMER_MSG_TIMEOUT = 60 * 60  # 1 Hour
 
-ADDITIONAL_STREAMER_UPLOAD_TIMEOUT = 10 * 60
+ADDITIONAL_STREAMER_UPLOAD_TIMEOUT = 3 * 60 * 60  # 3 hours
 
 
 def get_global_experiment():
@@ -108,6 +109,7 @@ PARSER_MAP = {
     float: parse_str_or_identity(float),
     bool: parse_str_or_identity(parse_bool),
     list: ParseListOf(str, parse_str_or_identity),
+    "int_list": ParseListOf(int, parse_str_or_identity(int)),
 }
 
 
@@ -127,11 +129,18 @@ class Config(object):
         self.config_map = config_map
         self.override = {}  # type: Dict[str, Any]
         self.backend_override = ConfigDictEnv({})
+        config_override = os.environ.get("COMET_INI")
+        if config_override is not None:
+            log_once_at_level(
+                logging.WARNING, "COMET_INI is deprecated; use COMET_CONFIG"
+            )
+        else:
+            config_override = os.environ.get("COMET_CONFIG")
         self.manager = ConfigManager(
             [  # User-defined overrides
                 ConfigOSEnv(),
                 ConfigEnvFileEnv(".env"),
-                ConfigIniEnv(os.environ.get("COMET_INI")),
+                ConfigIniEnv(config_override),
                 ConfigIniEnv("./.comet.config"),
                 ConfigIniEnv("~/.comet.config"),
                 # Comet-defined overrides
@@ -269,6 +278,145 @@ class Config(object):
 
         return value
 
+    def get_int_list(self, user_value, config_name, default=None, not_set_value=None):
+        # type: (Any, str, Optional[int], int) -> List[int]
+        """
+        Returns the correct config value based on the following priority list:
+        * User_value if set and not None
+        * The override value from the Backend
+        * The configured value
+        * The default value passed in argument if not None
+        * The configured value default
+
+        In addition make sure the returned value is a list of int
+        """
+
+        value = self.get_raw(
+            user_value=user_value,
+            config_name=config_name,
+            default=default,
+            not_set_value=not_set_value,
+        )
+
+        return value
+
+    def get_string_list(
+        self, user_value, config_name, default=None, not_set_value=None
+    ):
+        # type: (Any, str, Optional[int], int) -> List[str]
+        """
+        Returns the correct config value based on the following priority list:
+        * User_value if set and not None
+        * The override value from the Backend
+        * The configured value
+        * The default value passed in argument if not None
+        * The configured value default
+
+        In addition make sure the returned value is a list of str
+        """
+
+        value = self.get_raw(
+            user_value=user_value,
+            config_name=config_name,
+            default=default,
+            not_set_value=not_set_value,
+        )
+
+        return value
+
+    def get_deprecated_raw(
+        self,
+        old_user_value,
+        old_config_name,
+        new_user_value,
+        new_config_name,
+        new_not_set_value=None,
+    ):
+        # type: (Any, str, Any, str, Any) -> Any
+        """
+        Returns the correct value for deprecated config values:
+        * New user value
+        * Old user value
+        * New config value
+        * Old config value
+        * New config default
+
+        Note: The old config default is not used and should be set to None
+        """
+        old_config_value = self.get_raw(None, old_config_name, default=NO_VALUE)
+
+        if new_user_value is not new_not_set_value:
+            if old_user_value:
+                LOGGER.warning(
+                    "Deprecated config key %r was set, but ignored as new config key %r is set",
+                    old_config_name,
+                    new_config_name,
+                )
+            elif old_config_value:
+                LOGGER.warning(
+                    "Deprecated config key %r was set in %r, but ignored as new config key %r is set",
+                    old_config_name,
+                    self.get_config_origin(old_config_name),
+                    new_config_name,
+                )
+            return new_user_value
+
+        # Deprecated parameter default value must be None
+        if old_user_value is not None:
+            LOGGER.warning(
+                "Config key %r is deprecated, please use %r instead",
+                old_config_name,
+                new_config_name,
+            )
+            return old_user_value
+
+        new_config_value = self.get_raw(None, new_config_name, default=NO_VALUE)
+        if new_config_value is not NO_VALUE:
+            return new_config_value
+
+        old_config_value = self.get_raw(None, old_config_name, default=NO_VALUE)
+        if old_config_value is not NO_VALUE:
+            LOGGER.warning(
+                "Config key %r is deprecated (was set in %r), please use %r instead",
+                old_config_name,
+                self.get_config_origin(old_config_name),
+                new_config_name,
+            )
+            return old_config_value
+
+        config_type = self.config_map[new_config_name].get("type", str)
+        parser = PARSER_MAP[config_type]
+        return parser(self.config_map[new_config_name].get("default", None))
+
+    def get_deprecated_bool(
+        self,
+        old_user_value,
+        old_config_name,
+        new_user_value,
+        new_config_name,
+        new_not_set_value=None,
+    ):
+        # type: (Any, str, Any, str, bool) -> bool
+        """
+        Returns the correct value for deprecated config values:
+        * New user value
+        * Old user value
+        * New config value
+        * Old config value
+        * New config default
+
+        Note: The old config default is not used and should be set to None
+        """
+        value = self.get_deprecated_raw(
+            old_user_value,
+            old_config_name,
+            new_user_value,
+            new_config_name,
+            new_not_set_value=new_not_set_value,
+        )
+
+        return value
+
     def get_subsections(self):
         """
         Return the subsection config names.
@@ -410,7 +558,7 @@ class Config(object):
                         ini_file.write(six.u("[%s]\n" % section))
                         last_section = section
                     if isinstance(value, list):
-                        value = ",".join(value)
+                        value = ",".join([str(v) for v in value])
                     ini_file.write(six.u("%s = %s\n" % (setting, value)))
         print(" done!")
 
@@ -435,13 +583,12 @@ CONFIG_MAP = {
     "comet.git_directory": {"type": str},
     "comet.offline_sampling_size": {"type": int, "default": 15000},
     "comet.url_override": {"type": str, "default": "https://www.comet.ml/clientlib/"},
-    "comet.optimizer_url": {"type": str, "default": "https://optimizer.comet.ml"},
+    "comet.optimizer_url": {"type": str, "default": "https://www.comet.ml/optimizer/"},
     "comet.ws_url_override": {"type": str, "default": None},
     "comet.predictor_url": {"type": str, "default": "https://predictor.comet.ml/"},
     "comet.experiment_key": {"type": str},
     "comet.project_name": {"type": str},
     "comet.workspace": {"type": str},
-    "comet.display_summary": {"type": bool, "default": None},  # deprecated
     "comet.display_summary_level": {"type": int, "default": 1},
     # Logging
     "comet.logging.file": {"type": str},
@@ -499,13 +646,31 @@ CONFIG_MAP = {
     "comet.auto_log.git_patch": {"type": bool},
     "comet.auto_log.graph": {"type": bool},
     "comet.auto_log.metrics": {"type": bool},
+    "comet.auto_log.figures": {"type": bool, "default": True},
     "comet.auto_log.output_logger": {"type": str},
     "comet.auto_log.parameters": {"type": bool},
-    "comet.auto_log.weights": {"type": bool},
+    "comet.auto_log.histogram_tensorboard": {"type": bool, "default": False},
+    "comet.auto_log.histogram_epoch_rate": {"type": int, "default": 1},
+    "comet.auto_log.histogram_weights": {"type": bool, "default": False},
+    "comet.auto_log.histogram_gradients": {"type": bool, "default": False},
+    "comet.auto_log.histogram_activations": {"type": bool, "default": False},
+    "comet.keras.histogram_name_prefix": {
+        "type": str,
+        "default": "{layer_num:0{max_digits}d}",
+    },
+    "comet.keras.histogram_activation_index_list": {"type": "int_list", "default": "0"},
+    "comet.keras.histogram_activation_layer_list": {"type": list, "default": "-1"},
+    "comet.keras.histogram_batch_size": {"type": int, "default": 1000},
+    "comet.keras.histogram_gradient_index_list": {"type": "int_list", "default": "0"},
+    "comet.keras.histogram_gradient_layer_list": {"type": list, "default": "-1"},
+    "comet.auto_log.metric_step_rate": {"type": int, "default": 10},
     "comet.auto_log.co2": {"type": bool},
     "comet.auto_log.tfma": {"type": bool, "default": False},
-    # Internals
+    # Internals:
     "comet.internal.reporting": {"type": bool, "default": True},
+    # Deprecated:
+    "comet.display_summary": {"type": bool, "default": None},
+    "comet.auto_log.weights": {"type": bool, "default": None},
 }
 
 
