@@ -7,7 +7,7 @@
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
 #  Sign up for free at http://www.comet.ml
-#  Copyright (C) 2015-2020 Comet ML INC
+#  Copyright (C) 2015-2021 Comet ML INC
 #  This file can not be copied and/or distributed without the express
 #  permission of Comet ML Inc.
 # *******************************************************
@@ -164,12 +164,13 @@ def get_env_details_message(env_blacklist, include_env=False):
     )
 
 
-def get_caller_source_code():
-    # type: () -> Optional[str]
-    """ Returns the source code from the first caller that isn't Comet
+def get_caller_file_path():
+    # type: () -> Optional[Tuple[str, str]]
+    """ Returns the module name and file path from the first caller that isn't in the blacklisted
+    list. If none is found, returns None
     """
     # Don't get source code from these libraries:
-    ignore_list = ["comet_ml", "mlflow", "transformers", "ludwig"]
+    ignore_list = ("comet_ml", "mlflow", "transformers", "ludwig", "pytorch_lightning")
 
     for frame in inspect.stack(context=1):
         module = inspect.getmodule(frame[0])
@@ -180,8 +181,7 @@ def get_caller_source_code():
                 continue
 
             filename = module.__file__.rstrip("cd")
-            with open(filename) as f:
-                return f.read()
+            return (module_name, filename)
 
     # We didn't find any matching module
     LOGGER.debug("Didn't find any source code module")
