@@ -7,7 +7,7 @@
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
 #  Sign up for free at http://www.comet.ml
-#  Copyright (C) 2015-2020 Comet ML INC
+#  Copyright (C) 2015-2021 Comet ML INC
 #  This file can not be copied and/or distributed without the express
 #  permission of Comet ML Inc.
 # *******************************************************
@@ -26,6 +26,7 @@ import zipfile
 
 import six
 from requests import Response
+from requests_toolbelt import MultipartEncoder
 
 from ._reporting import FILE_UPLOADED_FAILED
 from ._typing import (
@@ -48,7 +49,7 @@ from .logging_messages import (
     UPLOAD_ASSET_TOO_BIG,
     UPLOAD_FILE_OS_ERROR,
 )
-from .messages import UploadFileMessage, UploadInMemoryMessage
+from .messages import RemoteAssetMessage, UploadFileMessage, UploadInMemoryMessage
 from .utils import (
     data_to_fp,
     encode_metadata,
@@ -79,17 +80,19 @@ def compress_git_patch(git_patch):
     return archive, zip_path
 
 
-def _send_file(url, files, params, experiment_id, config):
-    # type: (str, Any, Dict[str, str], str, Config) -> Response
-    LOGGER.debug("Uploading files %r to %s with params %s", files, url, params)
+def _send_file(url, data, params, experiment_id, config, headers):
+    # type: (str, Any, Dict[str, str], str, Config, Dict[str, Any]) -> Response
+    LOGGER.debug("Uploading files %r to %s with params %s", data, url, params)
+
+    headers.update({"X-COMET-DEBUG-EXPERIMENT-KEY": experiment_id})
 
     with get_http_session() as session:
         r = session.post(
             url,
             params=params,
-            files=files,
+            data=data,
             timeout=config["comet.timeout.file_upload"],
-            headers={"X-COMET-DEBUG-EXPERIMENT-KEY": experiment_id},
+            headers=headers,
         )  # type: Response
 
     LOGGER.debug("Uploading file to %s done", url)
@@ -119,19 +122,22 @@ def send_file(
         params.update(additional_params)
 
     with open(file_path, "rb") as _file:
-        files = {"file": _file}  # type: Dict[str, Any]
+        fields = {"file": ("file", _file)}  # type: Dict[str, Any]
 
         if metadata is not None:
             encoded_metadata = encode_metadata(metadata)
             if encoded_metadata:
-                files["metadata"] = encoded_metadata
+                fields["metadata"] = encoded_metadata
+
+        data = MultipartEncoder(fields=fields)
 
         return _send_file(
             post_endpoint,
             params=params,
-            files=files,
+            data=data,
             experiment_id=experiment_id,
             config=config,
+            headers={"Content-Type": data.content_type},
         )
 
 
@@ -151,18 +157,57 @@ def send_file_like(
     if additional_params is not None:
         params.update(additional_params)
 
-    files = {"file": file_like}
+    fields = {"file": ("file", file_like)}  # type: Dict[str, Any]
+
     if metadata is not None:
         encoded_metadata = encode_metadata(metadata)
         if encoded_metadata:
-            files["metadata"] = encoded_metadata
+            fields["metadata"] = encoded_metadata
+
+    data = MultipartEncoder(fields=fields)
 
     return _send_file(
         post_endpoint,
         params=params,
-        files=files,
+        data=data,
         experiment_id=experiment_id,
         config=config,
+        headers={"Content-Type": data.content_type},
+    )
+
+
+def send_remote_asset(
+    post_endpoint,
+    api_key,
+    experiment_id,
+    project_id,
+    remote_uri,
+    config,
+    additional_params=None,
+    metadata=None,
+):
+    # type: (str, str, str, str, str, Config, Optional[Dict[str, str]], Optional[Dict[Any, Any]]) -> Response
+    params = {"experimentId": experiment_id, "projectId": project_id, "apiKey": api_key}
+
+    if additional_params is not None:
+        params.update(additional_params)
+
+    fields = {"link": ("link", remote_uri)}  # type: Dict[str, Any]
+
+    if metadata is not None:
+        encoded_metadata = encode_metadata(metadata)
+        if encoded_metadata:
+            fields["metadata"] = encoded_metadata
+
+    data = MultipartEncoder(fields=fields)
+
+    return _send_file(
+        post_endpoint,
+        params=params,
+        data=data,
+        experiment_id=experiment_id,
+        config=config,
+        headers={"Content-Type": data.content_type},
     )
 
 
@@ -254,6 +299,46 @@ def upload_file_like(
         )
 
 
+def upload_remote_asset(
+    project_id,
+    experiment_id,
+    remote_uri,
+    upload_endpoint,
+    api_key,
+    additional_params=None,
+    metadata=None,
+):
+    config = get_config()
+
+    try:
+        response = send_remote_asset(
+            upload_endpoint,
+            api_key,
+            experiment_id,
+            project_id,
+            remote_uri,
+            config,
+            additional_params=additional_params,
+            metadata=metadata,
+        )
+
+        LOGGER.debug(
+            "Remote Asset successfully uploaded to (%s): %s",
+            response.status_code,
+            upload_endpoint,
+        )
+    except Exception as e:
+        LOGGER.error("Remote Asset could not be uploaded", exc_info=True)
+        Reporting.report(
+            event_name=FILE_UPLOADED_FAILED,
+            experiment_key=experiment_id,
+            project_id=project_id,
+            api_key=api_key,
+            err_msg=str(e),
+            config=config,
+        )
+
+
 def upload_file_thread(*args, **kwargs):
     p = threading.Thread(target=upload_file, args=args, kwargs=kwargs)
     p.daemon = True
@@ -268,9 +353,16 @@ def upload_file_like_thread(*args, **kwargs):
     return p
 
 
+def upload_remote_asset_thread(*args, **kwargs):
+    p = threading.Thread(target=upload_remote_asset, args=args, kwargs=kwargs)
+    p.daemon = True
+    p.start()
+    return p
+
+
 def is_valid_file_path(file_path):
     # type: (Any) -> bool
-    """ Check if the given argument is corresponding to a valid file path,
+    """Check if the given argument is corresponding to a valid file path,
     ready for reading
     """
     try:
@@ -293,7 +385,8 @@ def is_user_text(input):
 
 
 def check_max_file_size(file_path, max_upload_size, too_big_msg):
-    """ Check if a file identified by its file path is bigger than the maximum
+    # type: (str, int, str) -> int
+    """Check if a file identified by its file path is bigger than the maximum
     allowed upload size. Raises FileIsTooBig if the file is greater than the
     upload limit.
     """
@@ -311,7 +404,7 @@ def check_max_file_size(file_path, max_upload_size, too_big_msg):
 
 def save_matplotlib_figure(figure=None):
     # type: (Optional[Any]) -> str
-    """ Try saving either the current global pyplot figure or the given one
+    """Try saving either the current global pyplot figure or the given one
     and return None in case of error.
     """
     # Get the right figure to upload
@@ -370,6 +463,7 @@ class BaseUploadProcessor(object):
         LOGGER.debug("%r created with %r", self, self.__dict__)
 
     def process(self):
+        # type: () -> Union[None, UploadInMemoryMessage, UploadFileMessage]
         if isinstance(self.user_input, ValidFilePath) or is_valid_file_path(
             self.user_input
         ):
@@ -608,6 +702,29 @@ class FigureUploadProcessor(BaseUploadProcessor):
     TOO_BIG_MSG = LOG_FIGURE_TOO_BIG
     UPLOAD_TYPE = "visualization"
 
+    def __init__(
+        self,
+        user_input,  # type: Any
+        upload_limit,  # type: int
+        url_params,  # type: Dict[str, Optional[Any]]
+        metadata,  # type: Optional[Dict[str, str]]
+        copy_to_tmp,  # type: bool
+        error_message_identifier,  # type: Any
+        tmp_dir,  # type: str
+        upload_type=None,  # type: Optional[str]
+    ):
+        super(FigureUploadProcessor, self).__init__(
+            user_input,
+            upload_limit,
+            url_params,
+            metadata,
+            copy_to_tmp,
+            error_message_identifier,
+            tmp_dir,
+        )
+        if upload_type is not None:
+            self.UPLOAD_TYPE = upload_type
+
     def process_upload_to_be_converted(self, user_input):
         # type: (Any) -> Optional[UploadFileMessage]
         try:
@@ -617,6 +734,8 @@ class FigureUploadProcessor(BaseUploadProcessor):
             # An error occurred
             return None
 
+        self.url_params["extension"] = get_file_extension(filename)
+
         return self._process_upload_by_filepath(TemporaryFilePath(filename))
 
 
@@ -865,3 +984,26 @@ class GitPatchUploadProcessor(BaseUploadProcessor):
     def process_upload_by_filepath(self, upload_filepath):
         # type: (ValidFilePath) -> Optional[UploadFileMessage]
         return self._process_upload_by_filepath(upload_filepath)
+
+
+class RemoteAssetUploadProcessor(object):
+    def __init__(
+        self,
+        remote_uri,  # type: Any
+        upload_type,  # type: str
+        url_params,  # type: Dict[str, Optional[Any]]
+        metadata,  # type: Optional[Dict[str, str]]
+    ):
+        self.remote_uri = remote_uri
+        self.upload_type = upload_type
+        self.url_params = url_params
+        self.metadata = metadata
+
+    def process(self):
+        # type: () -> RemoteAssetMessage
+        # Add the expected params in the URL
+        self.url_params.update({"isRemote": True})
+
+        return RemoteAssetMessage(
+            self.remote_uri, self.upload_type, self.url_params, self.metadata
+        )
