@@ -7,7 +7,7 @@
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
 #  Sign up for free at http://www.comet.ml
-#  Copyright (C) 2015-2020 Comet ML INC
+#  Copyright (C) 2015-2021 Comet ML INC
 #  This file can not be copied and/or distributed without the express
 #  permission of Comet ML Inc.
 # *******************************************************
@@ -24,7 +24,10 @@ import time
 
 from comet_ml.utils import to_utf8
 
-import pynvml
+try:
+    import pynvml
+except ImportError:
+    pynvml = None
 
 LOGGER = logging.getLogger(__name__)
 
@@ -227,11 +230,14 @@ def get_gpu_details():
 
 
 def is_gpu_details_available():
-    try:
-        pynvml.nvmlInit()
-        return True
+    if pynvml is not None:
+        try:
+            pynvml.nvmlInit()
+            return True
 
-    except pynvml.NVMLError:
+        except pynvml.NVMLError:
+            return False
+    else:
         return False
 
 
