@@ -7,7 +7,7 @@
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
 #  Sign up for free at http://www.comet.ml
-#  Copyright (C) 2015-2020 Comet ML INC
+#  Copyright (C) 2015-2021 Comet ML INC
 #  This file can not be copied and/or distributed without the express
 #  permission of Comet ML Inc.
 # *******************************************************
@@ -29,9 +29,6 @@ def _get_loss(loss_backward, experiment):
     backward has been called. The right loss might be the unscaled one when
     using APEX or Pytorch 1.6 AMP
     """
-    experiment._localstorage.__dict__.setdefault("pytorch", {}).setdefault(
-        "amp_loss_mapping", {}
-    )
     return experiment._localstorage.pytorch["amp_loss_mapping"].get(
         id(loss_backward), loss_backward
     )
@@ -40,18 +37,12 @@ def _get_loss(loss_backward, experiment):
 def _add_amp_loss_mapping(scaled_loss, unscaled_loss, experiment):
     # type: (int, Any, BaseExperiment) -> None
     # First argument is the scaled_loss id
-    experiment._localstorage.__dict__.setdefault("pytorch", {}).setdefault(
-        "amp_loss_mapping", {}
-    )
     experiment._localstorage.pytorch["amp_loss_mapping"][scaled_loss] = unscaled_loss
 
 
 def _clean_amp_loss_mapping(scaled_loss, experiment):
     # type: (int, BaseExperiment) -> None
     # First argument is the scaled_loss id
-    experiment._localstorage.__dict__.setdefault("pytorch", {}).setdefault(
-        "amp_loss_mapping", {}
-    )
     experiment._localstorage.pytorch["amp_loss_mapping"].pop(scaled_loss, None)
 
 
@@ -63,7 +54,7 @@ def tensor_backward(experiment, original, result, *args, **kwargs):
         else:
             experiment.curr_step += 1
         if experiment.log_graph:
-            model = experiment._storage["torch"].get("model", None)
+            model = experiment._storage["torch"]["model"]
             if experiment.curr_step == 0 and model is not None:
                 experiment._set_model_graph(model, framework="pytorch")
         if experiment.auto_metric_logging:
@@ -74,11 +65,9 @@ def tensor_backward(experiment, original, result, *args, **kwargs):
                 loss = _get_loss(arg_loss, experiment)
 
                 if len(loss.data.shape) == 0:
+                    metric = loss.data.item()
                     experiment._log_metric(
-                        "loss",
-                        loss.data.item(),
-                        step=experiment.curr_step,
-                        framework="pytorch",
+                        "loss", metric, step=experiment.curr_step, framework="pytorch",
                     )
                 else:
                     experiment._log_metric(
@@ -99,7 +88,7 @@ def tensor_backward(experiment, original, result, *args, **kwargs):
 def model_constructor(experiment, original, *args, **kwargs):
     ## Assume the first one is the model:
     try:
-        model = experiment._storage["torch"].get("model", None)
+        model = experiment._storage["torch"]["model"]
         if model is None:
             experiment._storage["torch"]["model"] = args[1]
     except Exception:
