@@ -7,7 +7,7 @@
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
 #  Sign up for free at http://www.comet.ml
-#  Copyright (C) 2015-2020 Comet ML INC
+#  Copyright (C) 2015-2021 Comet ML INC
 #  This file can not be copied and/or distributed without
 #  the express permission of Comet ML Inc.
 # *******************************************************
@@ -195,6 +195,23 @@ class UploadInMemoryMessage(BaseMessage):
         return self.__dict__
 
 
+class RemoteAssetMessage(BaseMessage):
+    def __init__(
+        self,
+        remote_uri,  # type: Any
+        upload_type,  # type: str
+        additional_params,  # type: Dict[str, Optional[Any]]
+        metadata,  # type: Optional[Dict[str, str]]
+    ):
+        self.remote_uri = remote_uri
+        self.upload_type = upload_type
+        self.additional_params = additional_params
+        self.metadata = metadata
+
+    def _non_null_dict(self):
+        return self.__dict__
+
+
 class OsPackagesMessage(BaseMessage):
     def __init__(self, os_packages):
         # type: (List[str]) -> None
