@@ -7,7 +7,7 @@
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
 #  Sign up for free at http://www.comet.ml
-#  Copyright (C) 2015-2020 Comet ML INC
+#  Copyright (C) 2015-2021 Comet ML INC
 #  This file can not be copied and/or distributed without the express
 #  permission of Comet ML Inc.
 # *******************************************************
@@ -136,14 +136,15 @@ class CometModuleFinder(object):
         if fullname not in self.patcher_functions:
             return
 
-        from importlib.machinery import ModuleSpec
-
         spec = self.pathfinder.find_spec(fullname, path, target)
 
         if not spec:
             return
 
-        return ModuleSpec(fullname, CustomFileLoader(spec.loader, fullname, self))
+        new_loader = CustomFileLoader(spec.loader, fullname, self)
+        spec.loader = new_loader
+
+        return spec
 
     def _get_module(self, fullname):
         splitted_name = fullname.split(".")
