@@ -7,7 +7,7 @@
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
 #  Sign up for free at http://www.comet.ml
-#  Copyright (C) 2015-2020 Comet ML INC
+#  Copyright (C) 2015-2021 Comet ML INC
 #  This file can not be copied and/or distributed without the express
 #  permission of Comet ML Inc.
 # *******************************************************
@@ -18,20 +18,19 @@ Author: Boris Feld
 This module contains the code related to offline feature
 
 """
+import io
 import json
 import logging
-import os
 import os.path
 import shutil
 import tempfile
 import threading
 import traceback
 import zipfile
-from os.path import dirname, join
+from os.path import join
 from zipfile import ZipFile
 
 from jsonschema import ValidationError
-from jsonschema.validators import validator_for
 
 from ._reporting import (
     EXPERIMENT_CREATED,
@@ -39,8 +38,9 @@ from ._reporting import (
     OFFLINE_INVALID_WS_MSG,
 )
 from ._typing import List, Optional
-from .comet import OfflineStreamer, format_url, generate_guid
-from .config import get_api_key, get_config
+from .comet import OfflineStreamer, format_url, generate_guid, is_valid_experiment_key
+from .compat_utils import json_dump
+from .config import get_api_key, get_config, get_previous_experiment
 from .connection import (
     RestServerConnection,
     WebSocketConnection,
@@ -56,7 +56,7 @@ from .exceptions import (
 )
 from .experiment import BaseExperiment
 from .feature_toggles import FeatureToggles
-from .file_uploader import upload_file_thread
+from .file_uploader import upload_file_thread, upload_remote_asset_thread
 from .logging_messages import (
     CLOUD_DETAILS_MSG_SENDING_ERROR,
     MODEL_GRAPH_MSG_SENDING_ERROR,
@@ -70,6 +70,16 @@ from .logging_messages import (
     OS_PACKAGE_MSG_SENDING_ERROR,
 )
 from .metrics import MetricsSampler
+from .schemas import (
+    get_cloud_details_msg_validator,
+    get_experiment_file_validator,
+    get_graph_msg_validator,
+    get_os_packages_msg_validator,
+    get_remote_file_msg_validator,
+    get_system_details_msg_validator,
+    get_upload_msg_validator,
+    get_ws_msg_validator,
+)
 from .utils import local_timestamp
 
 LOGGER = logging.getLogger(__name__)
@@ -97,12 +107,19 @@ class OfflineExperiment(BaseExperiment):
         display_summary=None,  # type: Optional[bool]
         log_env_cpu=True,  # type: Optional[bool]
         display_summary_level=None,  # type: Optional[int]
-        auto_weight_logging=False,  # type: Optional[bool]
+        auto_weight_logging=None,  # type: Optional[bool]
         auto_log_co2=False,  # type: Optional[bool]
+        auto_metric_step_rate=10,  # type: Optional[int]
+        auto_histogram_tensorboard_logging=False,  # type: Optional[bool]
+        auto_histogram_epoch_rate=1,  # type: Optional[int]
+        auto_histogram_weight_logging=False,  # type: Optional[bool]
+        auto_histogram_gradient_logging=False,  # type: Optional[bool]
+        auto_histogram_activation_logging=False,  # type: Optional[bool]
     ):
         # type: (...) -> None
         """
-        Creates a new experiment on the Comet.ml frontend.
+        Creates a new experiment and serialize it on disk. The experiment file will need to be
+        upload manually later to appears on the frontend.
         Args:
             project_name: Optional. Send your experiment to a specific project. Otherwise will be sent to `Uncategorized Experiments`.
                              If project name does not already exists Comet.ml will create a new project.
@@ -111,7 +128,12 @@ class OfflineExperiment(BaseExperiment):
             log_graph: Default(True) - allows you to enable/disable automatic computation graph logging.
             auto_param_logging: Default(True) - allows you to enable/disable hyper parameters logging
             auto_metric_logging: Default(True) - allows you to enable/disable metrics logging
-            auto_weight_logging: Default(False) - allows you to enable/disable automatic histogram logging of biases and weights
+            auto_metric_step_rate: Default(10) - controls how often batch metrics are logged
+            auto_histogram_tensorboard_logging: Default(False) - allows you to enable/disable automatic histogram logging
+            auto_histogram_epoch_rate: Default(1) - controls how often histograms are logged
+            auto_histogram_weight_logging: Default(False) - allows you to enable/disable automatic histogram logging of biases and weights
+            auto_histogram_gradient_logging: Default(False) - allows you to enable/disable automatic histogram logging of gradients
+            auto_histogram_activation_logging: Default(False) - allows you to enable/disable automatic histogram logging of activations
             auto_output_logging: Default("default") - allows you to select
                 which output logging mode to use. You can pass `"native"`
                 which will log all output even when it originated from a C
@@ -164,6 +186,7 @@ class OfflineExperiment(BaseExperiment):
         # Start and ends time
         self.start_time = None
         self.stop_time = None
+        self.mode = "create"
 
         super(OfflineExperiment, self).__init__(
             project_name=project_name,
@@ -180,10 +203,17 @@ class OfflineExperiment(BaseExperiment):
             disabled=disabled,
             log_env_gpu=log_env_gpu,
             log_env_host=log_env_host,
+            display_summary=display_summary,
             display_summary_level=display_summary_level,
             log_env_cpu=log_env_cpu,
             auto_weight_logging=auto_weight_logging,
             auto_log_co2=auto_log_co2,
+            auto_metric_step_rate=auto_metric_step_rate,
+            auto_histogram_epoch_rate=auto_histogram_epoch_rate,
+            auto_histogram_tensorboard_logging=auto_histogram_tensorboard_logging,
+            auto_histogram_weight_logging=auto_histogram_weight_logging,
+            auto_histogram_gradient_logging=auto_histogram_gradient_logging,
+            auto_histogram_activation_logging=auto_histogram_activation_logging,
         )
 
         if not self.disabled:
@@ -237,9 +267,10 @@ class OfflineExperiment(BaseExperiment):
             "stop_time": self.stop_time,
             "tags": self.get_tags(),
             "workspace": self.workspace,
+            "mode": self.mode,
         }
-        with open(meta_file_path, "w") as f:
-            json.dump(meta, f)
+        with open(meta_file_path, "wb") as f:
+            json_dump(meta, f)
 
     def _get_offline_archive(self, directory, name):
         # ZIP the saved information
@@ -250,7 +281,7 @@ class OfflineExperiment(BaseExperiment):
             pass
 
         file_path = self._offline_zip_path(directory, name)
-        return ZipFile(file_path, "w")
+        return ZipFile(file_path, "w", allowZip64=True)
 
     def _mark_as_ended(self):
         if not self.alive:
@@ -318,58 +349,122 @@ class OfflineExperiment(BaseExperiment):
         return "[OfflineExperiment will get URL after upload]"
 
 
-def get_validator(filename, allow_additional_properties=True):
-    with open(join(dirname(__file__), join("schemas", filename))) as schema_file:
-        schema = json.load(schema_file)
+class ExistingOfflineExperiment(OfflineExperiment):
+    def __init__(
+        self,
+        project_name=None,  # type: Optional[str]
+        workspace=None,  # type: Optional[str]
+        log_code=True,  # type: Optional[bool]
+        log_graph=True,  # type: Optional[bool]
+        auto_param_logging=True,  # type: Optional[bool]
+        auto_metric_logging=True,  # type: Optional[bool]
+        parse_args=True,  # type: Optional[bool]
+        auto_output_logging="default",  # type: Optional[str]
+        log_env_details=True,  # type: Optional[bool]
+        log_git_metadata=True,  # type: Optional[bool]
+        log_git_patch=True,  # type: Optional[bool]
+        disabled=False,  # type: Optional[bool]
+        offline_directory=None,  # type: Optional[str]
+        log_env_gpu=True,  # type: Optional[bool]
+        log_env_host=True,  # type: Optional[bool]
+        api_key=None,  # type: Optional[str]
+        display_summary=None,  # type: Optional[bool]
+        log_env_cpu=True,  # type: Optional[bool]
+        display_summary_level=None,  # type: Optional[int]
+        auto_weight_logging=False,  # type: Optional[bool]
+        previous_experiment=None,  # type: Optional[str]
+    ):
+        # type: (...) -> None
+        """
+        Continue a previous experiment (identified by previous_experment) and serialize it on disk.
+        The experiment file will need to be upload manually later to append new information to the
+        previous experiment. The previous experiment need to exists before upload of the
+        ExistingOfflineExperiment.
+        Args:
+            previous_experiment: Optional. Your experiment key from comet.ml, could be set through
+                configuration as well.
+            project_name: Optional. Send your experiment to a specific project. Otherwise will be sent to `Uncategorized Experiments`.
+                             If project name does not already exists Comet.ml will create a new project.
+            workspace: Optional. Attach an experiment to a project that belongs to this workspace
+            log_code: Default(True) - allows you to enable/disable code logging
+            log_graph: Default(True) - allows you to enable/disable automatic computation graph logging.
+            auto_param_logging: Default(True) - allows you to enable/disable hyper parameters logging
+            auto_metric_logging: Default(True) - allows you to enable/disable metrics logging
+            parse_args: Default(True) - allows you to enable/disable automatic parsing of CLI arguments
+            auto_output_logging: Default("default") - allows you to select
+                which output logging mode to use. You can pass `"native"`
+                which will log all output even when it originated from a C
+                native library. You can also pass `"simple"` which will work
+                only for output made by Python code. If you want to disable
+                automatic output logging, you can pass `False`. The default is
+                `"default"` which will detect your environment and deactivate
+                the output logging for IPython and Jupyter environment and
+                sets `"native"` in the other cases.
+            log_env_details: Default(True) - log various environment
+                information in order to identify where the script is running
+            log_env_gpu: Default(True) - allow you to enable/disable the
+                automatic collection of gpu details and metrics (utilization, memory usage etc..).
+                `log_env_details` must also be true.
+            log_env_cpu: Default(True) - allow you to enable/disable the
+                automatic collection of cpu details and metrics (utilization, memory usage etc..).
+                `log_env_details` must also be true.
+            log_env_host: Default(True) - allow you to enable/disable the
+                automatic collection of host information (ip, hostname, python version, user etc...).
+                `log_env_details` must also be true.
+            log_git_metadata: Default(True) - allow you to enable/disable the
+                automatic collection of git details
+            display_summary_level: Default(1) - control the summary detail that is
+                displayed on the console at end of experiment. If 0, the summary
+                notification is still sent. Valid values are 0 to 2.
+            disabled: Default(False) - allows you to disable all network
+                communication with the Comet.ml backend. It is useful when you
+                want to test to make sure everything is working, without actually
+                logging anything.
+            offline_directory: the directory used to save the offline archive
+                for the experiment.
+        """
+        self.config = get_config()
 
-    if not allow_additional_properties:
-        schema["additionalProperties"] = False
+        self.previous_experiment = get_previous_experiment(
+            previous_experiment, self.config
+        )
 
-    validator_class = validator_for(schema)
-    validator_class.check_schema(schema)
-    return validator_class(schema)
+        if not is_valid_experiment_key(self.previous_experiment):
+            raise ValueError("Invalid experiment key: %s" % self.previous_experiment)
 
+        super(ExistingOfflineExperiment, self).__init__(
+            project_name=project_name,
+            workspace=workspace,
+            log_code=log_code,
+            log_graph=log_graph,
+            auto_param_logging=auto_param_logging,
+            auto_metric_logging=auto_metric_logging,
+            parse_args=parse_args,
+            auto_output_logging=auto_output_logging,
+            log_env_details=log_env_details,
+            log_git_metadata=log_git_metadata,
+            log_git_patch=log_git_patch,
+            disabled=disabled,
+            offline_directory=offline_directory,
+            log_env_gpu=log_env_gpu,
+            log_env_host=log_env_host,
+            api_key=api_key,
+            log_env_cpu=log_env_cpu,
+            display_summary=display_summary,
+            display_summary_level=display_summary_level,
+            auto_weight_logging=auto_weight_logging,
+        )
 
-def get_experiment_file_validator(allow_additional_properties=True):
-    return get_validator("offline-experiment.schema.json", allow_additional_properties)
+        self.mode = "append"
 
-
-def get_ws_msg_validator(allow_additional_properties=True):
-    return get_validator("offline-ws-msg.schema.json", allow_additional_properties)
-
-
-def get_os_packages_msg_validator(allow_additional_properties=False):
-    return get_validator(
-        "offline-os-packages-msg.schema.json", allow_additional_properties
-    )
-
-
-def get_graph_msg_validator(allow_additional_properties=False):
-    return get_validator("offline-graph-msg.schema.json", allow_additional_properties)
-
-
-def get_system_details_msg_validator(allow_additional_properties=False):
-    return get_validator(
-        "offline-system-details-msg.schema.json", allow_additional_properties
-    )
-
-
-def get_cloud_details_msg_validator(allow_additional_properties=False):
-    return get_validator(
-        "offline-cloud-details-msg.schema.json", allow_additional_properties
-    )
-
-
-def get_upload_msg_validator(allow_additional_properties=True):
-    return get_validator(
-        "offline-file-upload-msg.schema.json", allow_additional_properties
-    )
+    def _get_experiment_key(self):
+        return self.previous_experiment
 
 
 def unzip_offline_archive(offline_archive_path):
     temp_dir = tempfile.mkdtemp()
 
-    zip_file = zipfile.ZipFile(offline_archive_path, mode="r")
+    zip_file = zipfile.ZipFile(offline_archive_path, mode="r", allowZip64=True)
 
     # Extract the archive
     zip_file.extractall(temp_dir)
@@ -401,6 +496,7 @@ class OfflineSender(object):
         self.system_details_msg_validator = get_system_details_msg_validator()
         self.cloud_details_msg_validator = get_cloud_details_msg_validator()
         self.upload_msg_validator = get_upload_msg_validator()
+        self.remote_file_msg_validator = get_remote_file_msg_validator()
 
         self.server_address = get_backend_address()
 
@@ -433,7 +529,9 @@ class OfflineSender(object):
         self._send_start_ends_time()
 
     def _read_experiment_file(self):
-        with open(join(self.offline_dir, "experiment.json")) as experiment_file:
+        with io.open(
+            join(self.offline_dir, "experiment.json"), encoding="utf-8"
+        ) as experiment_file:
             metadata = json.load(experiment_file)
 
         self.experiment_file_validator.validate(metadata)
@@ -453,11 +551,17 @@ class OfflineSender(object):
         self.stop_time = metadata["stop_time"]
         self.tags = metadata["tags"]
         self.workspace = metadata["workspace"]
+        self.mode = metadata.get("mode", "create")
 
     def _handshake(self):
-        run_id_results = self.connection.get_run_id(
-            self.project_name, self.workspace, offline=True
-        )
+        if self.mode == "create":
+            run_id_results = self.connection.get_run_id(
+                self.project_name, self.workspace, offline=True
+            )
+        elif self.mode == "append":
+            run_id_results = self.connection.get_old_run_id(self.experiment_id)
+        else:
+            raise ValueError("Unknown mode value %r" % self.mode)
 
         (
             self.run_id,
@@ -497,7 +601,9 @@ class OfflineSender(object):
 
         sampler = MetricsSampler(sampling_size)
 
-        with open(join(self.offline_dir, "messages.json")) as messages_files:
+        with io.open(
+            join(self.offline_dir, "messages.json"), encoding="utf-8"
+        ) as messages_files:
             for i, line in enumerate(messages_files):
                 try:
                     message = json.loads(line)
@@ -518,6 +624,8 @@ class OfflineSender(object):
                             self._process_ws_msg(message_payload)
                     elif message_type == "file_upload":
                         self._process_upload_message(message)
+                    elif message_type == "remote_file":
+                        self._process_remote_file_message(message)
                     elif message_type == "os_packages":
                         self._process_os_packages_message(message)
                     elif message_type == "graph":
@@ -595,6 +703,38 @@ class OfflineSender(object):
         LOGGER.debug("Processing uploading message done")
         LOGGER.debug("Upload threads %s", self.upload_threads)
 
+    def _process_remote_file_message(self, message):
+        message = message["payload"]
+
+        try:
+            self.remote_file_msg_validator.validate(message)
+        except ValidationError:
+            if self.validation_error:
+                raise
+
+            tb = traceback.format_exc()
+            LOGGER.warning(OFFLINE_EXPERIMENT_INVALID_UPLOAD_MSG, exc_info=True)
+            self.connection.report(event_name=OFFLINE_INVALID_UPLOAD_MSG, err_msg=tb)
+
+        # Compute the url from the upload type
+        url = self.connection.get_upload_url(message["upload_type"])
+
+        additional_params = message["additional_params"] or {}
+        additional_params["runId"] = self.run_id
+
+        upload_thread = upload_remote_asset_thread(
+            project_id=self.project_id,
+            experiment_id=self.experiment_id,
+            remote_uri=message["remote_uri"],
+            upload_endpoint=url,
+            api_key=self.api_key,
+            additional_params=additional_params,
+            metadata=message["metadata"],
+        )
+        self.upload_threads.append(upload_thread)
+        LOGGER.debug("Processing uploading message done")
+        LOGGER.debug("Upload threads %s", self.upload_threads)
+
     def _process_os_packages_message(self, message):
         message = message["payload"]
 
@@ -727,9 +867,18 @@ class OfflineSender(object):
         )
 
     def _send_start_ends_time(self):
-        self.connection.offline_experiment_start_end_time(
-            self.run_id, self.start_time, self.stop_time
-        )
+        if self.mode == "create":
+            self.connection.offline_experiment_start_end_time(
+                self.run_id, self.start_time, self.stop_time
+            )
+        elif self.mode == "append":
+            self.connection.offline_experiment_start_end_time(
+                self.run_id, None, self.stop_time
+            )
+        else:
+            # This shouldn't happen in the normal flow as the invalid mode is validated during
+            # handshake
+            raise ValueError("Invalid offline mode %r" % self.mode)
 
     def _get_experiment_url(self):
         if self.focus_link:
