@@ -8,7 +8,7 @@
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
 #  Sign up for free at http://www.comet.ml
-#  Copyright (C) 2015-2020 Comet ML INC
+#  Copyright (C) 2015-2021 Comet ML INC
 #  This file can not be copied and/or distributed without
 #  the express permission of Comet ML Inc.
 # *******************************************************
@@ -31,7 +31,11 @@ Examples:
     comet offline 60a1a617e4c24c8998cc78fa3bc7a31b.zip
     comet offline --csv 60a1a617e4c24c8998cc78fa3bc7a31b.zip
 
-    comet troubleshoot
+    comet check
+
+    comet init
+
+    comet models
 
 Note that `comet optimize` requires your COMET_API_KEY
 be configured in the environment, or in your .comet.config
