@@ -8,7 +8,7 @@
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
 #  Sign up for free at http://www.comet.ml
-#  Copyright (C) 2015-2020 Comet ML INC
+#  Copyright (C) 2015-2021 Comet ML INC
 #  This file can not be copied and/or distributed without
 #  the express permission of Comet ML Inc.
 # *******************************************************
@@ -174,10 +174,13 @@ class ArchivedExperiment(object):
                     "os_packages",
                     "system_details",
                     "cloud_details",
+                    "remote_file",
                 ]:
                     continue
                 else:
-                    raise Exception("Unknown message type: %s" % message["type"])
+                    LOGGER.error(
+                        "Unknown message  type: %s", message["type"], exc_info=True
+                    )
 
     def process_payload(self, payload):
         """
